<?php

namespace Agi\Action;

use Agi\ChannelInfo;
use Agi\Wrapper;
use Ivoz\Core\Domain\Service\EntityTools;
use Ivoz\Provider\Domain\Model\FaxesInOut\FaxesInOutDto;
use Ivoz\Provider\Domain\Model\FaxesInOut\FaxesInOutInterface;

/**
 * @class ExternalFaxCallAction
 *
 * @brief Manage outgoing external calls generated by faxes
 *
 */
class ExternalFaxCallAction
{
    /**
     * @var Wrapper
     */
    protected $agi;

    /**
     * @var ChannelInfo
     */
    protected $channelInfo;

    /**
     * @var EntityTools
     */
    protected $entityTools;

    /**
     * @var string
     */
    protected $number;

    /**
     * @var FaxesInOutInterface
     */
    protected $faxfile;

    /**
     * ExternalFaxCallAction constructor.
     * @param Wrapper $agi
     * @param ChannelInfo $channelInfo
     * @param EntityTools $entityTools
     */
    public function __construct(
        Wrapper $agi,
        ChannelInfo $channelInfo,
        EntityTools $entityTools
    ) {
        $this->entityTools = $entityTools;
        $this->agi = $agi;
        $this->channelInfo = $channelInfo;
    }

    /**
     * @param FaxesInOutInterface|null $faxfile
     * @return $this
     */
    public function setFaxfile(FaxesInOutInterface $faxfile = null)
    {
        $this->faxfile = $faxfile;
        return $this;
    }

    /**
     * @param string|null $number
     * @return $this
     */
    public function setDestination(string $number = null)
    {
        $this->number = $number;
        return $this;
    }

    public function process()
    {
        $agent = $this->channelInfo->getChannelCaller();
        $faxOut = $this->faxfile;
        $number = $this->number;

        // Some feedback for asterisk cli
        $this->agi->notice("Processing External call from <green>%s</green> to %s", $agent, $number);

        // Get Outgoing presentation
        $ddi = $agent->getOutgoingDDI($number);

        // We need Outgoing DDI for external call presentation
        if (!$ddi) {
            $this->agi->error("%s has not OutgoingDDI configured", $agent);
            $this->agi->decline();


            /** @var FaxesInOutDto $faxOutDto */
            $faxOutDto = $this->entityTools->entityToDto($faxOut);
            $faxOutDto->setStatus('error');
            $this->entityTools->persistDto($faxOutDto, $faxOut);

            return;
        }

        // Set fax
        $this->agi->setVariable("__FAXID", $faxOut->getFax()->getId());

        // Set ddi
        $this->agi->setVariable("__DDIID", $ddi->getId());

        // Set presentation
        $this->agi->setCallerIdNum($ddi->getDdie164());

        // Call the PSJIP endpoint
        $this->agi->setVariable("__DIAL_DST", "PJSIP/" . $number . '@proxytrunks');
        $this->agi->setVariable("__DIAL_OPTS", "");

        $this->agi->redirect('faxes-call-world', $number);
    }
}
